@@ -315,7 +315,7 @@ class DynamicsFunctions:
         DynamicsFunctions.apply_parameters(parameters, nlp)
         q, qdot, residual_tau = DynamicsFunctions.dispatch_q_qdot_tau_data(states, controls, nlp)
 
-        muscles_states = biorbd.VecBiorbdMuscleState(nlp.shape["muscle"])
+        muscles_states = nlp.model.stateSet()
         muscles_activations = controls[nlp.shape["tau"] :]
 
         for k in range(nlp.shape["muscle"]):
@@ -357,7 +357,7 @@ class DynamicsFunctions:
         DynamicsFunctions.apply_parameters(parameters, nlp)
         q, qdot, residual_tau = DynamicsFunctions.dispatch_q_qdot_tau_data(states, controls, nlp)
 
-        muscles_states = biorbd.VecBiorbdMuscleState(nlp.shape["muscle"])
+        muscles_states = nlp.model.stateSet()
         muscles_activations = controls[nlp.shape["tau"] :]
 
         for k in range(nlp.shape["muscle"]):
@@ -400,7 +400,7 @@ class DynamicsFunctions:
         DynamicsFunctions.apply_parameters(parameters, nlp)
         q, qdot, residual_tau = DynamicsFunctions.dispatch_q_qdot_tau_data(states, controls, nlp)
 
-        muscles_states = biorbd.VecBiorbdMuscleState(nlp.shape["muscle"])
+        muscles_states = nlp.model.stateSet()
         muscles_activations = controls[nlp.shape["tau"] :]
 
         for k in range(nlp.shape["muscle"]):
@@ -442,7 +442,7 @@ class DynamicsFunctions:
         q = nlp.mapping["q"].to_second.map(states[:nq])
         qdot = nlp.mapping["qdot"].to_second.map(states[nq:])
 
-        muscles_states = biorbd.VecBiorbdMuscleState(nlp.shape["muscle"])
+        muscles_states = nlp.model.stateSet()
         muscles_activations = controls
 
         for k in range(nlp.shape["muscle"]):
@@ -484,7 +484,7 @@ class DynamicsFunctions:
         q = nlp.mapping["q"].to_second.map(states[:nq])
         qdot = nlp.mapping["qdot"].to_second.map(states[nq:])
 
-        muscles_states = biorbd.VecBiorbdMuscleState(nlp.shape["muscle"])
+        muscles_states = nlp.model.stateSet()
         muscles_excitation = controls
         muscles_activations = states[nlp.shape["q"] + nlp.shape["qdot"] :]
 
@@ -528,7 +528,7 @@ class DynamicsFunctions:
         DynamicsFunctions.apply_parameters(parameters, nlp)
         q, qdot, residual_tau = DynamicsFunctions.dispatch_q_qdot_tau_data(states, controls, nlp)
 
-        muscles_states = biorbd.VecBiorbdMuscleState(nlp.shape["muscle"])
+        muscles_states = nlp.model.stateSet()
         muscles_excitation = controls[nlp.shape["tau"] :]
         muscles_activations = states[nlp.shape["q"] + nlp.shape["qdot"] :]
 
@@ -573,7 +573,7 @@ class DynamicsFunctions:
         DynamicsFunctions.apply_parameters(parameters, nlp)
         q, qdot, residual_tau = DynamicsFunctions.dispatch_q_qdot_tau_data(states, controls, nlp)
 
-        muscles_states = biorbd.VecBiorbdMuscleState(nlp.shape["muscle"])
+        muscles_states = nlp.model.stateSet()
         muscles_excitation = controls[nlp.shape["tau"] :]
         muscles_activations = states[nlp.shape["q"] + nlp.shape["qdot"] :]
 
@@ -618,7 +618,7 @@ class DynamicsFunctions:
         DynamicsFunctions.apply_parameters(parameters, nlp)
         q, qdot, residual_tau = DynamicsFunctions.dispatch_q_qdot_tau_data(states, controls, nlp)
 
-        muscles_states = biorbd.VecBiorbdMuscleState(nlp.shape["muscle"])
+        muscles_states = nlp.model.stateSet()
         muscles_excitation = controls[nlp.shape["tau"] :]
         muscles_activations = states[nlp.shape["q"] + nlp.shape["qdot"] :]
 
@@ -632,6 +632,33 @@ class DynamicsFunctions:
         biorbd.Model.ForwardDynamicsConstraintsDirect(nlp.model, q, qdot, tau, cs)
         return cs.getForce().to_mx()
 
+    @staticmethod
+    def dispatch_q_qdot_data(states: MX.sym, controls: MX.sym, nlp) -> tuple:
+        """
+        Extracting q and qdot from states and controls, assuming state, state and control, respectively.
+
+        Parameters
+        ----------
+        states: MX.sym
+            The state of the system
+        controls: MX.sym
+            The controls of the system
+        nlp: NonLinearProgram
+            The definition of the system
+
+        Returns
+        ----------
+        MX.sym
+            q, the generalized coordinates
+        MX.sym
+            qdot, the generalized velocities
+        """
+
+        nq = nlp.mapping["q"].to_first.len
+        q = nlp.mapping["q"].to_second.map(states[:nq])
+        qdot = nlp.mapping["qdot"].to_second.map(states[nq:])
+        return q, qdot
+
     @staticmethod
     def dispatch_q_qdot_tau_data(states: MX.sym, controls: MX.sym, nlp) -> tuple:
         """
@@ -656,11 +683,8 @@ class DynamicsFunctions:
             tau, the generalized torques
         """
 
-        nq = nlp.mapping["q"].to_first.len
-        q = nlp.mapping["q"].to_second.map(states[:nq])
-        qdot = nlp.mapping["qdot"].to_second.map(states[nq:])
+        q, qdot = DynamicsFunctions.dispatch_q_qdot_data(states, controls, nlp)
         tau = nlp.mapping["tau"].to_second.map(controls[: nlp.shape["tau"]])
-
         return q, qdot, tau
 
     @staticmethod
@@ -676,7 +700,9 @@ class DynamicsFunctions:
             The definition of the system
         """
 
+        offset = 0
         for param in nlp.parameters:
             # Call the pre dynamics function
             if param.function:
-                param.function(nlp.model, parameters, **param.params)
+                param.function(nlp.model, parameters[offset : offset + param.size], **param.params)
+                offset += param.size
